@@ -1,5 +1,5 @@
 /* Hooks for cfg representation specific functions.
-   Copyright (C) 2003-2022 Free Software Foundation, Inc.
+   Copyright (C) 2003-2024 Free Software Foundation, Inc.
    Contributed by Sebastian Pop <s.pop@laposte.net>
 
 This file is part of GCC.
@@ -18,6 +18,7 @@ You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING3.  If not see
 <http://www.gnu.org/licenses/>.  */
 
+#define INCLUDE_MEMORY
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
@@ -102,7 +103,7 @@ DEBUG_FUNCTION void
 verify_flow_info (void)
 {
   size_t *edge_checksum;
-  int err = 0;
+  bool err = false;
   basic_block bb, last_bb_seen;
   basic_block *last_visited;
 
@@ -118,14 +119,14 @@ verify_flow_info (void)
 	  && bb != BASIC_BLOCK_FOR_FN (cfun, bb->index))
 	{
 	  error ("bb %d on wrong place", bb->index);
-	  err = 1;
+	  err = true;
 	}
 
       if (bb->prev_bb != last_bb_seen)
 	{
 	  error ("prev_bb of %d should be %d, not %d",
 		 bb->index, last_bb_seen->index, bb->prev_bb->index);
-	  err = 1;
+	  err = true;
 	}
 
       last_bb_seen = bb;
@@ -142,18 +143,18 @@ verify_flow_info (void)
 	{
 	  error ("verify_flow_info: Block %i has loop_father, but there are no loops",
 		 bb->index);
-	  err = 1;
+	  err = true;
 	}
       if (bb->loop_father == NULL && current_loops != NULL)
 	{
 	  error ("verify_flow_info: Block %i lacks loop_father", bb->index);
-	  err = 1;
+	  err = true;
 	}
 
       if (!bb->count.verify ())
 	{
 	  error ("verify_flow_info: Wrong count of block %i", bb->index);
-	  err = 1;
+	  err = true;
 	}
       /* FIXME: Graphite and SLJL and target code still tends to produce
 	 edges with no probability.  */
@@ -161,13 +162,13 @@ verify_flow_info (void)
           && !bb->count.initialized_p () && !flag_graphite && 0)
 	{
 	  error ("verify_flow_info: Missing count of block %i", bb->index);
-	  err = 1;
+	  err = true;
 	}
 
       if (bb->flags & ~cfun->cfg->bb_flags_allocated)
 	{
 	  error ("verify_flow_info: unallocated flag set on BB %d", bb->index);
-	  err = 1;
+	  err = true;
 	}
 
       FOR_EACH_EDGE (e, ei, bb->succs)
@@ -176,7 +177,7 @@ verify_flow_info (void)
 	    {
 	      error ("verify_flow_info: Duplicate edge %i->%i",
 		     e->src->index, e->dest->index);
-	      err = 1;
+	      err = true;
 	    }
 	  /* FIXME: Graphite and SLJL and target code still tends to produce
 	     edges with no probability.  */
@@ -185,13 +186,13 @@ verify_flow_info (void)
 	    {
 	      error ("Uninitialized probability of edge %i->%i", e->src->index,
 		     e->dest->index);
-	      err = 1;
+	      err = true;
 	    }
 	  if (!e->probability.verify ())
 	    {
 	      error ("verify_flow_info: Wrong probability of edge %i->%i",
 		     e->src->index, e->dest->index);
-	      err = 1;
+	      err = true;
 	    }
 
 	  last_visited [e->dest->index] = bb;
@@ -208,14 +209,14 @@ verify_flow_info (void)
 	      fprintf (stderr, "\nSuccessor: ");
 	      dump_edge_info (stderr, e, TDF_DETAILS, 1);
 	      fprintf (stderr, "\n");
-	      err = 1;
+	      err = true;
 	    }
 
 	  if (e->flags & ~cfun->cfg->edge_flags_allocated)
 	    {
 	      error ("verify_flow_info: unallocated edge flag set on %d -> %d",
 		     e->src->index, e->dest->index);
-	      err = 1;
+	      err = true;
 	    }
 
 	  edge_checksum[e->dest->index] += (size_t) e;
@@ -223,7 +224,7 @@ verify_flow_info (void)
       if (n_fallthru > 1)
 	{
 	  error ("wrong amount of branch edges after unconditional jump %i", bb->index);
-	  err = 1;
+	  err = true;
 	}
 
       FOR_EACH_EDGE (e, ei, bb->preds)
@@ -236,7 +237,7 @@ verify_flow_info (void)
 	      fputs ("\nSuccessor: ", stderr);
 	      dump_edge_info (stderr, e, TDF_DETAILS, 1);
 	      fputc ('\n', stderr);
-	      err = 1;
+	      err = true;
 	    }
 
 	  if (ei.index != e->dest_idx)
@@ -249,7 +250,7 @@ verify_flow_info (void)
 	      fputs ("\nSuccessor: ", stderr);
 	      dump_edge_info (stderr, e, TDF_DETAILS, 1);
 	      fputc ('\n', stderr);
-	      err = 1;
+	      err = true;
 	    }
 
 	  edge_checksum[e->dest->index] -= (size_t) e;
@@ -272,7 +273,7 @@ verify_flow_info (void)
     if (edge_checksum[bb->index])
       {
 	error ("basic block %i edge lists are corrupted", bb->index);
-	err = 1;
+	err = true;
       }
 
   /* Clean up.  */
@@ -280,7 +281,9 @@ verify_flow_info (void)
   free (edge_checksum);
 
   if (cfg_hooks->verify_flow_info)
-    err |= cfg_hooks->verify_flow_info ();
+    if (cfg_hooks->verify_flow_info ())
+      err = true;
+
   if (err)
     internal_error ("verify_flow_info failed");
   timevar_pop (TV_CFG_VERIFY);
@@ -542,7 +545,6 @@ split_block_1 (basic_block bb, void *i)
     return NULL;
 
   new_bb->count = bb->count;
-  new_bb->discriminator = bb->discriminator;
 
   if (dom_info_available_p (CDI_DOMINATORS))
     {
@@ -1087,9 +1089,16 @@ can_duplicate_block_p (const_basic_block bb)
   return cfg_hooks->can_duplicate_block_p (bb);
 }
 
-/* Duplicates basic block BB and redirects edge E to it.  Returns the
-   new basic block.  The new basic block is placed after the basic block
-   AFTER.  */
+/* Duplicate basic block BB, place it after AFTER (if non-null) and redirect
+   edge E to it (if non-null).  Return the new basic block.
+
+   If BB contains a returns_twice call, the caller is responsible for recreating
+   incoming abnormal edges corresponding to the "second return" for the copy.
+   gimple_can_duplicate_bb_p rejects such blocks, while RTL likes to live
+   dangerously.
+
+   If BB has incoming abnormal edges for some other reason, their destinations
+   should be tied to label(s) of the original BB and not the copy.  */
 
 basic_block
 duplicate_block (basic_block bb, edge e, basic_block after, copy_bb_data *id)
