@@ -1,5 +1,5 @@
 /* Machine description for AArch64 architecture.
-   Copyright (C) 2009-2025 Free Software Foundation, Inc.
+   Copyright (C) 2009-2024 Free Software Foundation, Inc.
    Contributed by ARM Ltd.
 
    This file is part of GCC.
@@ -759,19 +759,6 @@ struct aarch64_required_extensions
 	     sm_on ? sm_on | flags : 0 };
   }
 
-  /* Return a requirement that is as restrictive as possible while still being
-     no more restrictive than THIS and no more restrictive than OTHER.  */
-  inline CONSTEXPR aarch64_required_extensions
-  common_denominator (const aarch64_required_extensions &other)
-  {
-    return { sm_off && other.sm_off
-	     ? sm_off & other.sm_off
-	     : sm_off | other.sm_off,
-	     sm_on && other.sm_on
-	     ? sm_on & other.sm_on
-	     : sm_on | other.sm_on };
-  }
-
   /* Require non-streaming mode and the features in FLAGS.  */
   static inline CONSTEXPR aarch64_required_extensions
   nonstreaming_only (aarch64_feature_flags flags)
@@ -853,11 +840,9 @@ bool aarch64_and_bitmask_imm (unsigned HOST_WIDE_INT val_in, machine_mode mode);
 int aarch64_branch_cost (bool, bool);
 enum aarch64_symbol_type aarch64_classify_symbolic_expression (rtx);
 bool aarch64_advsimd_struct_mode_p (machine_mode mode);
-opt_machine_mode aarch64_v64_mode (scalar_mode);
-opt_machine_mode aarch64_v128_mode (scalar_mode);
+opt_machine_mode aarch64_vq_mode (scalar_mode);
 opt_machine_mode aarch64_full_sve_mode (scalar_mode);
 bool aarch64_can_const_movi_rtx_p (rtx x, machine_mode mode);
-bool aarch64_valid_fp_move (rtx, rtx, machine_mode);
 bool aarch64_const_vec_all_same_int_p (rtx, HOST_WIDE_INT);
 bool aarch64_const_vec_all_same_in_range_p (rtx, HOST_WIDE_INT,
 					    HOST_WIDE_INT);
@@ -866,9 +851,8 @@ bool aarch64_rnd_imm_p (rtx);
 bool aarch64_constant_address_p (rtx);
 bool aarch64_emit_approx_div (rtx, rtx, rtx);
 bool aarch64_emit_approx_sqrt (rtx, rtx, bool);
-bool aarch64_emit_opt_vec_rotate (rtx, rtx, rtx);
 tree aarch64_vector_load_decl (tree);
-rtx aarch64_gen_callee_cookie (aarch64_isa_mode, arm_pcs, bool);
+rtx aarch64_gen_callee_cookie (aarch64_isa_mode, arm_pcs);
 void aarch64_expand_call (rtx, rtx, rtx, bool);
 bool aarch64_expand_cpymem_mops (rtx *, bool);
 bool aarch64_expand_cpymem (rtx *, bool);
@@ -897,8 +881,6 @@ bool aarch64_move_imm (unsigned HOST_WIDE_INT, machine_mode);
 machine_mode aarch64_sve_int_mode (machine_mode);
 opt_machine_mode aarch64_sve_pred_mode (unsigned int);
 machine_mode aarch64_sve_pred_mode (machine_mode);
-opt_machine_mode aarch64_advsimd_vector_array_mode (machine_mode,
-						    unsigned HOST_WIDE_INT);
 opt_machine_mode aarch64_sve_data_mode (scalar_mode, poly_uint64);
 bool aarch64_sve_mode_p (machine_mode);
 HOST_WIDE_INT aarch64_fold_sve_cnt_pat (aarch64_svpattern, unsigned int);
@@ -1021,7 +1003,6 @@ void aarch64_expand_mov_immediate (rtx, rtx);
 rtx aarch64_stack_protect_canary_mem (machine_mode, rtx, aarch64_salt_type);
 rtx aarch64_ptrue_reg (machine_mode);
 rtx aarch64_ptrue_reg (machine_mode, unsigned int);
-rtx aarch64_ptrue_reg (machine_mode, machine_mode);
 rtx aarch64_pfalse_reg (machine_mode);
 bool aarch64_sve_same_pred_for_ptest_p (rtx *, rtx *);
 void aarch64_emit_sve_pred_move (rtx, rtx, rtx);
@@ -1031,7 +1012,6 @@ rtx aarch64_replace_reg_mode (rtx, machine_mode);
 void aarch64_split_sve_subreg_move (rtx, rtx, rtx);
 void aarch64_expand_prologue (void);
 void aarch64_expand_vector_init (rtx, rtx);
-void aarch64_sve_expand_vector_init_subvector (rtx, rtx);
 void aarch64_sve_expand_vector_init (rtx, rtx);
 void aarch64_init_cumulative_args (CUMULATIVE_ARGS *, const_tree, rtx,
 				   const_tree, unsigned, bool = false);
@@ -1064,7 +1044,7 @@ rtx aarch64_simd_expand_builtin (int, tree, rtx);
 void aarch64_simd_lane_bounds (rtx, HOST_WIDE_INT, HOST_WIDE_INT, const_tree);
 rtx aarch64_endian_lane_rtx (machine_mode, unsigned int);
 
-void aarch64_split_move (rtx, rtx, machine_mode);
+void aarch64_split_double_move (rtx, rtx, machine_mode);
 void aarch64_split_128bit_move (rtx, rtx);
 
 bool aarch64_split_128bit_move_p (rtx, rtx);
@@ -1098,7 +1078,8 @@ void aarch64_finish_ldpstp_peephole (rtx *, bool,
 				     enum rtx_code = (enum rtx_code)0);
 
 void aarch64_expand_sve_vec_cmp_int (rtx, rtx_code, rtx, rtx);
-void aarch64_expand_sve_vec_cmp_float (rtx, rtx_code, rtx, rtx);
+bool aarch64_expand_sve_vec_cmp_float (rtx, rtx_code, rtx, rtx, bool);
+void aarch64_expand_sve_vcond (machine_mode, machine_mode, rtx *);
 
 bool aarch64_prepare_sve_int_fma (rtx *, rtx_code);
 bool aarch64_prepare_sve_cond_int_fma (rtx *, rtx_code);
@@ -1124,18 +1105,6 @@ bool aarch64_general_check_builtin_call (location_t, vec<location_t>,
 					 unsigned int, tree, unsigned int,
 					 tree *);
 
-namespace aarch64 {
-  void report_non_ice (location_t, tree, unsigned int);
-  void report_out_of_range (location_t, tree, unsigned int, HOST_WIDE_INT,
-			    HOST_WIDE_INT, HOST_WIDE_INT);
-  void report_neither_nor (location_t, tree, unsigned int, HOST_WIDE_INT,
-			   HOST_WIDE_INT, HOST_WIDE_INT);
-  void report_not_one_of (location_t, tree, unsigned int, HOST_WIDE_INT,
-			  HOST_WIDE_INT, HOST_WIDE_INT, HOST_WIDE_INT,
-			  HOST_WIDE_INT);
-  void report_not_enum (location_t, tree, unsigned int, HOST_WIDE_INT, tree);
-}
-
 namespace aarch64_sve {
   void init_builtins ();
   void handle_arm_sve_h (bool);
@@ -1155,8 +1124,6 @@ namespace aarch64_sve {
 #ifdef GCC_TARGET_H
   bool verify_type_context (location_t, type_context_kind, const_tree, bool);
 #endif
- void add_sve_type_attribute (tree, unsigned int, unsigned int,
-			      const char *, const char *);
 }
 
 extern void aarch64_split_combinev16qi (rtx operands[3]);
@@ -1190,35 +1157,19 @@ void aarch64_set_asm_isa_flags (aarch64_feature_flags);
 void aarch64_set_asm_isa_flags (gcc_options *, aarch64_feature_flags);
 bool aarch64_handle_option (struct gcc_options *, struct gcc_options *,
 			     const struct cl_decoded_option *, location_t);
-void aarch64_print_hint_for_extensions (const char *);
-void aarch64_print_hint_for_arch (const char *);
-void aarch64_print_hint_for_core (const char *);
+const char *aarch64_rewrite_selected_cpu (const char *name);
 enum aarch_parse_opt_result aarch64_parse_extension (const char *,
                                                      aarch64_feature_flags *,
                                                      std::string *);
-enum aarch_parse_opt_result aarch64_parse_arch (const char *,
-						aarch64_arch *,
-						aarch64_feature_flags *,
-						std::string *);
-enum aarch_parse_opt_result aarch64_parse_cpu (const char *,
-					       aarch64_cpu *,
-					       aarch64_feature_flags *,
-					       std::string *);
-enum aarch_parse_opt_result aarch64_parse_tune (const char *, aarch64_cpu *);
-bool aarch64_validate_march (const char *, aarch64_arch *,
-			     aarch64_feature_flags *);
-bool aarch64_validate_mcpu (const char *, aarch64_cpu *,
-			    aarch64_feature_flags *);
-bool aarch64_validate_mtune (const char *, aarch64_cpu *);
+void aarch64_get_all_extension_candidates (auto_vec<const char *> *candidates);
 std::string aarch64_get_extension_string_for_isa_flags (aarch64_feature_flags,
 							aarch64_feature_flags);
-std::string aarch64_get_arch_string_for_assembler (aarch64_arch,
-						   aarch64_feature_flags);
 
 rtl_opt_pass *make_pass_aarch64_early_ra (gcc::context *);
 rtl_opt_pass *make_pass_fma_steering (gcc::context *);
 rtl_opt_pass *make_pass_track_speculation (gcc::context *);
 rtl_opt_pass *make_pass_late_track_speculation (gcc::context *);
+rtl_opt_pass *make_pass_tag_collision_avoidance (gcc::context *);
 rtl_opt_pass *make_pass_insert_bti (gcc::context *ctxt);
 rtl_opt_pass *make_pass_cc_fusion (gcc::context *ctxt);
 rtl_opt_pass *make_pass_switch_pstate_sm (gcc::context *ctxt);
@@ -1254,13 +1205,5 @@ extern void aarch64_adjust_reg_alloc_order ();
 
 bool aarch64_optimize_mode_switching (aarch64_mode_entity);
 void aarch64_restore_za (rtx);
-void aarch64_expand_crc_using_pmull (scalar_mode, scalar_mode, rtx *);
-void aarch64_expand_reversed_crc_using_pmull (scalar_mode, scalar_mode, rtx *);
-
-
-extern bool aarch64_gcs_enabled ();
-
-extern unsigned aarch64_data_alignment (const_tree exp, unsigned align);
-extern unsigned aarch64_stack_alignment (const_tree exp, unsigned align);
 
 #endif /* GCC_AARCH64_PROTOS_H */
