@@ -3,7 +3,7 @@
    building RTL.  These routines are used both during actual parsing
    and during the instantiation of template functions.
 
-   Copyright (C) 1998-2025 Free Software Foundation, Inc.
+   Copyright (C) 1998-2024 Free Software Foundation, Inc.
 
    This file is part of GCC.
 
@@ -21,6 +21,7 @@ You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING3.  If not see
 <http://www.gnu.org/licenses/>.  */
 
+#define INCLUDE_MEMORY
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
@@ -32,7 +33,6 @@ along with GCC; see the file COPYING3.  If not see
 #include "gimplify.h"
 #include "target.h"
 #include "decl.h"
-#include "flags.h"
 
 /* Constructor for a lambda expression.  */
 
@@ -613,7 +613,7 @@ add_capture (tree lambda, tree id, tree orig_init, bool by_reference_p,
 	    return error_mark_node;
 	}
 
-      if (cxx_dialect < cxx20 && !explicit_init_p)
+      if (cxx_dialect < cxx20)
 	{
 	  auto_diagnostic_group d;
 	  tree stripped_init = tree_strip_any_location_wrapper (initializer);
@@ -786,12 +786,6 @@ lambda_expr_this_capture (tree lambda, int add_capture_p)
   tree result;
 
   tree this_capture = LAMBDA_EXPR_THIS_CAPTURE (lambda);
-  if (this_capture)
-    if (tree spec = retrieve_local_specialization (this_capture))
-      {
-	gcc_checking_assert (generic_lambda_fn_p (lambda_function (lambda)));
-	this_capture = spec;
-      }
 
   /* In unevaluated context this isn't an odr-use, so don't capture.  */
   if (cp_unevaluated_operand)
@@ -1045,17 +1039,15 @@ nonlambda_method_basetype (void)
     }
 }
 
-/* Like current_scope, but looking through lambdas.  If ONLY_SKIP_CLOSURES_P,
-   only look through closure types.  */
+/* Like current_scope, but looking through lambdas.  */
 
 tree
-current_nonlambda_scope (bool only_skip_closures_p/*=false*/)
+current_nonlambda_scope (void)
 {
   tree scope = current_scope ();
   for (;;)
     {
-      if (!only_skip_closures_p
-	  && TREE_CODE (scope) == FUNCTION_DECL
+      if (TREE_CODE (scope) == FUNCTION_DECL
 	  && LAMBDA_FUNCTION_P (scope))
 	{
 	  scope = CP_TYPE_CONTEXT (DECL_CONTEXT (scope));
@@ -1549,43 +1541,13 @@ finish_lambda_scope (void)
 void
 record_lambda_scope (tree lambda)
 {
-  tree closure = LAMBDA_EXPR_CLOSURE (lambda);
-  gcc_checking_assert (closure);
-
-  /* Before ABI v20, lambdas in static data member initializers did not
-     get a dedicated lambda scope.  */
-  tree scope = lambda_scope.scope;
-  if (is_static_data_member_initialized_in_class (scope))
+  LAMBDA_EXPR_EXTRA_SCOPE (lambda) = lambda_scope.scope;
+  if (lambda_scope.scope)
     {
-      if (!abi_version_at_least (20))
-	scope = NULL_TREE;
-      if (warn_abi && abi_version_crosses (20) && !processing_template_decl)
-	{
-	  if (abi_version_at_least (20))
-	    warning_at (location_of (closure), OPT_Wabi,
-			"the mangled name of %qT changed in "
-			"%<-fabi-version=20%> (GCC 15.1)", closure);
-	  else
-	    warning_at (location_of (closure), OPT_Wabi,
-			"the mangled name of %qT changes in "
-			"%<-fabi-version=20%> (GCC 15.1)", closure);
-	}
+      tree closure = LAMBDA_EXPR_CLOSURE (lambda);
+      gcc_checking_assert (closure);
+      maybe_key_decl (lambda_scope.scope, TYPE_NAME (closure));
     }
-
-  /* An otherwise unattached class-scope lambda in a member template
-     should not have a mangling scope, as the mangling scope will not
-     correctly inherit on instantiation.  */
-  tree ctx = TYPE_CONTEXT (closure);
-  if (scope
-      && ctx
-      && CLASS_TYPE_P (ctx)
-      && ctx == TREE_TYPE (scope)
-      && current_template_depth > template_class_depth (ctx))
-    scope = NULL_TREE;
-
-  LAMBDA_EXPR_EXTRA_SCOPE (lambda) = scope;
-  if (scope)
-    maybe_key_decl (scope, TYPE_NAME (closure));
 }
 
 // Compare lambda template heads TMPL_A and TMPL_B, used for both
