@@ -1,5 +1,5 @@
 /* d-builtins.cc -- GCC builtins support for D.
-   Copyright (C) 2006-2025 Free Software Foundation, Inc.
+   Copyright (C) 2006-2024 Free Software Foundation, Inc.
 
 GCC is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@ You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING3.  If not see
 <http://www.gnu.org/licenses/>.  */
 
+#define INCLUDE_MEMORY
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
@@ -139,8 +140,8 @@ build_frontend_type (tree type)
 	  dtype = Type::basic[i];
 
 	  /* Search for type matching size and signedness.  */
-	  if (unsignedp != dtype->isUnsigned ()
-	      || size != dmd::size (dtype))
+	  if (unsignedp != dtype->isunsigned ()
+	      || size != dtype->size ())
 	    continue;
 
 	  return dmd::addMod (dtype, mod);
@@ -157,7 +158,7 @@ build_frontend_type (tree type)
 	  dtype = Type::basic[i];
 
 	  /* Search for type matching size.  */
-	  if (dmd::size (dtype) != size)
+	  if (dtype->size () != size)
 	    continue;
 
 	  return dmd::addMod (dtype, mod);
@@ -174,7 +175,7 @@ build_frontend_type (tree type)
 	  dtype = Type::basic[i];
 
 	  /* Search for type matching size.  */
-	  if (dmd::size (dtype) != size)
+	  if (dtype->size () != size)
 	    continue;
 
 	  return dmd::addMod (dtype, mod);
@@ -215,7 +216,7 @@ build_frontend_type (tree type)
 	break;
 
       dtype = dmd::addMod (dmd::sarrayOf (dtype, nunits), mod);
-      if (target.isVectorTypeSupported (dmd::size (dtype), dtype->nextOf ()))
+      if (target.isVectorTypeSupported (dtype->size (), dtype->nextOf ()))
 	break;
 
       dtype = dmd::addMod (TypeVector::create (dtype), mod);
@@ -522,7 +523,6 @@ d_init_versions (void)
   targetdm.d_cpu_versions ();
   targetdm.d_os_versions ();
 
-  VersionCondition::addPredefinedGlobalIdent ("CppRuntime_GNU");
   VersionCondition::addPredefinedGlobalIdent ("CppRuntime_Gcc");
 }
 
@@ -572,8 +572,8 @@ d_build_builtins_module (Module *m)
       tf->trust = !DECL_ASSEMBLER_NAME_SET_P (decl) ? TRUST::safe
 	: TREE_NOTHROW (decl) ? TRUST::trusted
 	: TRUST::system;
-      tf->isNothrow (true);
-      tf->isNogc (true);
+      tf->isnothrow (true);
+      tf->isnogc (true);
 
       FuncDeclaration *func
 	= FuncDeclaration::create (Loc (), Loc (),
@@ -712,11 +712,11 @@ matches_builtin_type (Type *t1, Type *t2)
 
   if (((tb1->isTypePointer () && tb2->isTypePointer ())
        || (tb1->isTypeVector () && tb2->isTypeVector ()))
-      && dmd::implicitConvTo (tb1, tb2) != MATCH::nomatch)
+      && tb1->implicitConvTo (tb2) != MATCH::nomatch)
     return true;
 
-  if (tb1->isIntegral () == tb2->isIntegral ()
-      && dmd::size (tb1) == dmd::size (tb2))
+  if (tb1->isintegral () == tb2->isintegral ()
+      && tb1->size () == tb2->size ())
     return true;
 
   return false;
@@ -739,7 +739,7 @@ covariant_with_builtin_type_p (Type *t1, Type *t2)
 
   /* Check for obvious reasons why types may be distinct.  */
   if (tf1 == NULL || tf2 == NULL
-      || tf1->isRef () != tf2->isRef ()
+      || tf1->isref () != tf2->isref ()
       || tf1->parameterList.varargs != tf2->parameterList.varargs
       || tf1->parameterList.length () != tf2->parameterList.length ())
     return false;
@@ -777,7 +777,7 @@ maybe_set_builtin_1 (Dsymbol *d)
   if (ad != NULL)
     {
       /* Recursively search through attribute decls.  */
-      Dsymbols *decls = dmd::include (ad, NULL);
+      Dsymbols *decls = ad->include (NULL);
       if (decls && decls->length)
 	{
 	  for (size_t i = 0; i < decls->length; i++)
