@@ -1,5 +1,5 @@
 /* d-lang.cc -- Language-dependent hooks for D.
-   Copyright (C) 2006-2025 Free Software Foundation, Inc.
+   Copyright (C) 2006-2024 Free Software Foundation, Inc.
 
 GCC is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@ You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING3.  If not see
 <http://www.gnu.org/licenses/>.  */
 
+#define INCLUDE_MEMORY
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
@@ -302,7 +303,7 @@ d_init_options (unsigned int, cl_decoded_option *decoded_options)
 
   /* Warnings and deprecations are disabled by default.  */
   global.params.useDeprecated = DIAGNOSTICinform;
-  global.params.useWarnings = DIAGNOSTICoff;
+  global.params.warnings = DIAGNOSTICoff;
   global.params.v.errorLimit = flag_max_errors;
   global.params.v.messageStyle = MessageStyle::gnu;
 
@@ -563,7 +564,6 @@ d_handle_option (size_t scode, const char *arg, HOST_WIDE_INT value,
       global.params.previewIn = value;
       global.params.fix16997 = value;
       global.params.noSharedAccess = FeatureState::enabled;
-      global.params.safer = FeatureState::enabled;
       global.params.rvalueRefParam = FeatureState::enabled;
       global.params.inclusiveInContracts = value;
       global.params.systemVariables = FeatureState::enabled;
@@ -614,10 +614,6 @@ d_handle_option (size_t scode, const char *arg, HOST_WIDE_INT value,
       global.params.noSharedAccess = FeatureState::enabled;
       break;
 
-    case OPT_fpreview_safer:
-      global.params.safer = FeatureState::enabled;
-      break;
-
     case OPT_fpreview_rvaluerefparam:
       global.params.rvalueRefParam = FeatureState::enabled;
       break;
@@ -777,7 +773,7 @@ d_handle_option (size_t scode, const char *arg, HOST_WIDE_INT value,
 
     case OPT_Wall:
       if (value)
-	global.params.useWarnings = DIAGNOSTICinform;
+	global.params.warnings = DIAGNOSTICinform;
       break;
 
     case OPT_Wdeprecated:
@@ -786,7 +782,7 @@ d_handle_option (size_t scode, const char *arg, HOST_WIDE_INT value,
 
     case OPT_Werror:
       if (value)
-	global.params.useWarnings = DIAGNOSTICerror;
+	global.params.warnings = DIAGNOSTICerror;
       break;
 
     case OPT_Wspeculative:
@@ -912,7 +908,7 @@ d_post_options (const char ** fn)
 
   /* Error about use of deprecated features.  */
   if (global.params.useDeprecated == DIAGNOSTICinform
-      && global.params.useWarnings == DIAGNOSTICerror)
+      && global.params.warnings == DIAGNOSTICerror)
     global.params.useDeprecated = DIAGNOSTICerror;
 
   if (flag_excess_precision == EXCESS_PRECISION_DEFAULT)
@@ -925,8 +921,7 @@ d_post_options (const char ** fn)
     global.params.v.errorLimit = flag_max_errors;
 
   global.params.v.showColumns = flag_show_column;
-  global.params.v.errorPrintMode = flag_diagnostics_show_caret
-    ? ErrorPrintMode::printErrorContext : ErrorPrintMode::simpleError;
+  global.params.v.printErrorContext = flag_diagnostics_show_caret;
 
   /* Keep the front-end location type in sync with params.  */
   Loc::set (global.params.v.showColumns, global.params.v.messageStyle);
@@ -939,12 +934,7 @@ d_post_options (const char ** fn)
   /* The front-end parser only has access to `compileEnv', synchronize its
      fields with params.  */
   global.compileEnv.previewIn = global.params.previewIn;
-  global.compileEnv.transitionIn = global.params.v.vin;
   global.compileEnv.ddocOutput = global.params.ddoc.doOutput;
-  global.compileEnv.cCharLookupTable =
-    IdentifierCharLookup::forTable (IdentifierTable::C11);
-  global.compileEnv.dCharLookupTable =
-    IdentifierCharLookup::forTable (IdentifierTable::LR);
 
   if (warn_return_type == -1)
     warn_return_type = 0;
@@ -1716,8 +1706,8 @@ d_types_compatible_p (tree x, tree y)
 	return true;
 
       /* Type system allows implicit conversion between.  */
-      if (dmd::implicitConvTo (tx, ty) != MATCH::nomatch
-	  || dmd::implicitConvTo (ty, tx) != MATCH::nomatch)
+      if (tx->implicitConvTo (ty) != MATCH::nomatch
+	  || ty->implicitConvTo (tx) != MATCH::nomatch)
 	return true;
     }
 
