@@ -1,5 +1,5 @@
 /* decl.cc -- Lower D frontend declarations to GCC trees.
-   Copyright (C) 2006-2025 Free Software Foundation, Inc.
+   Copyright (C) 2006-2024 Free Software Foundation, Inc.
 
 GCC is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@ You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING3.  If not see
 <http://www.gnu.org/licenses/>.  */
 
+#define INCLUDE_MEMORY
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
@@ -36,7 +37,6 @@ along with GCC; see the file COPYING3.  If not see
 #include "dmd/nspace.h"
 #include "dmd/target.h"
 #include "dmd/template.h"
-#include "dmd/typinf.h"
 
 #include "tree.h"
 #include "tree-iterator.h"
@@ -256,18 +256,18 @@ public:
 
   void visit (Module *d) final override
   {
-    if (d->semanticRun () >= PASS::obj)
+    if (d->semanticRun >= PASS::obj)
       return;
 
     build_module_tree (d);
-    d->semanticRun (PASS::obj);
+    d->semanticRun = PASS::obj;
   }
 
   /* Write the imported symbol to debug.  */
 
   void visit (Import *d) final override
   {
-    if (d->semanticRun () >= PASS::obj)
+    if (d->semanticRun >= PASS::obj)
       return;
 
     /* Implements import declarations by telling the debug back-end we are
@@ -320,7 +320,7 @@ public:
 					      false, false);
       }
 
-    d->semanticRun (PASS::obj);
+    d->semanticRun = PASS::obj;
   }
 
   /* Finish a top-level `asm` definition.  */
@@ -351,7 +351,7 @@ public:
 
   void visit (AttribDeclaration *d) final override
   {
-    Dsymbols *ds = dmd::include (d, NULL);
+    Dsymbols *ds = d->include (NULL);
 
     if (!ds)
       return;
@@ -472,7 +472,7 @@ public:
 
   void visit (StructDeclaration *d) final override
   {
-    if (d->semanticRun () >= PASS::obj)
+    if (d->semanticRun >= PASS::obj)
       return;
 
     if (d->type->ty == TY::Terror)
@@ -520,7 +520,7 @@ public:
     if (d->xhash)
       this->build_dsymbol (d->xhash);
 
-    d->semanticRun (PASS::obj);
+    d->semanticRun = PASS::obj;
   }
 
   /* Finish semantic analysis of functions in vtbl for class CD.  */
@@ -542,7 +542,7 @@ public:
 	  has_errors = true;
 
 	/* No name hiding to check for.  */
-	if (!dmd::isFuncHidden (d, fd) || fd->isFuture ())
+	if (!d->isFuncHidden (fd) || fd->isFuture ())
 	  continue;
 
 	/* The function fd is hidden from the view of the class.
@@ -562,8 +562,10 @@ public:
 	    if (fd2->isFuture ())
 	      continue;
 
-	    if (dmd::leastAsSpecialized (fd, fd2, NULL) == MATCH::nomatch
-		&& dmd::leastAsSpecialized (fd2, fd, NULL) == MATCH::nomatch)
+	    if (FuncDeclaration::leastAsSpecialized (fd, fd2, NULL)
+		    == MATCH::nomatch
+		&& FuncDeclaration::leastAsSpecialized (fd2, fd, NULL)
+		    == MATCH::nomatch)
 	      continue;
 
 	    /* Hiding detected; same name, overlapping specializations.  */
@@ -588,7 +590,7 @@ public:
 
   void visit (ClassDeclaration *d) final override
   {
-    if (d->semanticRun () >= PASS::obj)
+    if (d->semanticRun >= PASS::obj)
       return;
 
     if (d->type->ty == TY::Terror)
@@ -617,7 +619,7 @@ public:
 
     /* Generate C symbols.  */
     d->csym = get_classinfo_decl (d);
-    Dsymbol *vtblsym = dmd::vtblSymbol (d);
+    Dsymbol *vtblsym = d->vtblSymbol ();
     vtblsym->csym = get_vtable_decl (d);
     tree sinit = aggregate_initializer_decl (d);
 
@@ -654,7 +656,7 @@ public:
       = build_constructor (TREE_TYPE (vtblsym->csym), elms);
     d_finish_decl (vtblsym->csym);
 
-    d->semanticRun (PASS::obj);
+    d->semanticRun = PASS::obj;
   }
 
   /* Write out compiler generated TypeInfo and vtables for the given interface
@@ -662,7 +664,7 @@ public:
 
   void visit (InterfaceDeclaration *d) final override
   {
-    if (d->semanticRun () >= PASS::obj)
+    if (d->semanticRun >= PASS::obj)
       return;
 
     if (d->type->ty == TY::Terror)
@@ -697,7 +699,7 @@ public:
     DECL_INITIAL (d->csym) = layout_classinfo (d);
     d_finish_decl (d->csym);
 
-    d->semanticRun (PASS::obj);
+    d->semanticRun = PASS::obj;
   }
 
   /* Write out compiler generated TypeInfo and initializer for the given
@@ -705,10 +707,10 @@ public:
 
   void visit (EnumDeclaration *d) final override
   {
-    if (d->semanticRun () >= PASS::obj)
+    if (d->semanticRun >= PASS::obj)
       return;
 
-    if (d->errors () || d->type->ty == TY::Terror)
+    if (d->errors || d->type->ty == TY::Terror)
       {
 	error_at (make_location_t (d->loc),
 		  "had semantic errors when compiling");
@@ -728,7 +730,7 @@ public:
       create_typeinfo (d->type, NULL);
 
     TypeEnum *tc = d->type->isTypeEnum ();
-    if (tc->sym->members && !dmd::isZeroInit (d->type))
+    if (tc->sym->members && !d->type->isZeroInit ())
       {
 	/* Generate static initializer.  */
 	d->sinit = enum_initializer_decl (d);
@@ -736,7 +738,7 @@ public:
 	d_finish_decl (d->sinit);
       }
 
-    d->semanticRun (PASS::obj);
+    d->semanticRun = PASS::obj;
   }
 
   /* Finish up a variable declaration and push it into the current scope.
@@ -744,7 +746,7 @@ public:
 
   void visit (VarDeclaration *d) final override
   {
-    if (d->semanticRun () >= PASS::obj)
+    if (d->semanticRun >= PASS::obj)
       return;
 
     if (d->type->ty == TY::Terror)
@@ -782,7 +784,7 @@ public:
       {
 	/* Do not store variables we cannot take the address of,
 	   but keep the values for purposes of debugging.  */
-	if (d->type->isScalar () && !dmd::hasPointers (d->type))
+	if (d->type->isscalar () && !dmd::hasPointers (d->type))
 	  {
 	    tree decl = get_symbol_decl (d);
 	    d_pushdecl (decl);
@@ -804,7 +806,7 @@ public:
 	  return;
 
 	/* How big a symbol can be should depend on back-end.  */
-	tree size = build_integer_cst (dmd::size (d->type, d->loc),
+	tree size = build_integer_cst (d->type->size (d->loc),
 				       build_ctype (Type::tsize_t));
 	if (!valid_constant_size_p (size))
 	  {
@@ -822,7 +824,7 @@ public:
 		DECL_INITIAL (decl) = build_expr (e, true);
 	      }
 	  }
-	else if (!dmd::isZeroInit (d->type))
+	else if (!d->type->isZeroInit ())
 	  {
 	    /* Use default initializer for the type.  */
 	    if (TypeStruct *ts = d->type->isTypeStruct ())
@@ -889,7 +891,7 @@ public:
 	  }
       }
 
-    d->semanticRun (PASS::obj);
+    d->semanticRun = PASS::obj;
   }
 
   /* Generate and compile a static TypeInfo declaration, but only if it is
@@ -897,16 +899,16 @@ public:
 
   void visit (TypeInfoDeclaration *d) final override
   {
-    if (d->semanticRun () >= PASS::obj)
+    if (d->semanticRun >= PASS::obj)
       return;
 
-    if (dmd::isSpeculativeType (d->tinfo))
+    if (speculative_type_p (d->tinfo))
       return;
 
     tree t = get_typeinfo_decl (d);
     DECL_INITIAL (t) = layout_typeinfo (d);
     d_finish_decl (t);
-    d->semanticRun (PASS::obj);
+    d->semanticRun = PASS::obj;
   }
 
   /* Finish up a function declaration and compile it all the way
@@ -915,7 +917,7 @@ public:
   void visit (FuncDeclaration *d) final override
   {
     /* Already generated the function.  */
-    if (d->semanticRun () >= PASS::obj)
+    if (d->semanticRun >= PASS::obj)
       return;
 
     /* Don't emit any symbols from gcc.attributes module.  */
@@ -957,7 +959,7 @@ public:
       }
 
     /* Ensure all semantic passes have run.  */
-    if (d->semanticRun () < PASS::semantic3)
+    if (d->semanticRun < PASS::semantic3)
       {
 	gcc_assert (!doing_semantic_analysis_p);
 
@@ -971,8 +973,8 @@ public:
       return;
 
     /* Start generating code for this function.  */
-    gcc_assert (d->semanticRun () == PASS::semantic3done);
-    d->semanticRun (PASS::obj);
+    gcc_assert (d->semanticRun == PASS::semantic3done);
+    d->semanticRun = PASS::obj;
 
     /* Duplicated FuncDeclarations map to the same symbol.  Check if this
        is the one declaration which will be emitted.  */
@@ -1115,7 +1117,7 @@ build_decl_tree (Dsymbol *d)
   if (d->loc.filename ())
     input_location = make_location_t (d->loc);
   else
-    input_location = make_location_t (Loc::singleFilename ("<no_file>"));
+    input_location = make_location_t (Loc ("<no_file>", 1, 0));
 
   DeclVisitor v = DeclVisitor ();
   v.build_dsymbol (d);
@@ -1179,7 +1181,7 @@ maybe_build_decl_tree (Declaration *decl)
 
   /* Still running semantic analysis on declaration, or it has already had its
      code generated.  */
-  if (doing_semantic_analysis_p || decl->semanticRun () >= PASS::obj)
+  if (doing_semantic_analysis_p || decl->semanticRun >= PASS::obj)
     return decl->csym;
 
   if (error_operand_p (decl->csym))
@@ -1246,7 +1248,7 @@ get_symbol_decl (Declaration *decl)
       /* CONST_DECL was initially intended for enumerals and may be used for
 	 scalars in general, but not for aggregates.  Here a non-constant
 	 value is generated anyway so as its value can be used.  */
-      if (!vd->canTakeAddressOf () && !vd->type->isScalar ())
+      if (!vd->canTakeAddressOf () && !vd->type->isscalar ())
 	{
 	  gcc_assert (vd->_init && !vd->_init->isVoidInitializer ());
 	  Expression *ie = dmd::initializerToExpression (vd->_init);
@@ -1307,7 +1309,7 @@ get_symbol_decl (Declaration *decl)
 	  /* Cannot make an expression out of a void initializer.  */
 	  gcc_assert (vd->_init && !vd->_init->isVoidInitializer ());
 	  /* Non-scalar manifest constants have already been dealt with.  */
-	  gcc_assert (vd->type->isScalar ());
+	  gcc_assert (vd->type->isscalar ());
 
 	  Expression *ie = dmd::initializerToExpression (vd->_init);
 	  DECL_INITIAL (decl->csym) = build_expr (ie, true);
@@ -1324,7 +1326,7 @@ get_symbol_decl (Declaration *decl)
       /* `const` applies to data that cannot be changed by the const reference
 	 to that data. It may, however, be changed by another reference to that
 	 same data.  */
-      if (vd->isConst () && !vd->isResult () && !vd->isDataseg ())
+      if (vd->isConst () && !vd->isDataseg ())
 	TREE_READONLY (decl->csym) = 1;
     }
 
@@ -2212,7 +2214,7 @@ get_vtable_decl (ClassDeclaration *decl)
      will have a different type.  However the back-end seems to accept this.  */
   tree type = build_ctype (dmd::sarrayOf (Type::tvoidptr, decl->vtbl.length));
 
-  Dsymbol *vtblsym = dmd::vtblSymbol (decl);
+  Dsymbol *vtblsym = decl->vtblSymbol ();
   vtblsym->csym = declare_extern_var (ident, type);
   DECL_LANG_SPECIFIC (vtblsym->csym) = build_lang_decl (NULL);
 
@@ -2393,12 +2395,6 @@ aggregate_initializer_decl (AggregateDeclaration *decl)
       SET_DECL_ALIGN (sinit, sd->alignment.get () * BITS_PER_UNIT);
       DECL_USER_ALIGN (sinit) = true;
     }
-  else if (sd == NULL)
-    {
-      /* Alignment of class is determined its biggest field alignment.  */
-      SET_DECL_ALIGN (sinit, decl->alignsize * BITS_PER_UNIT);
-      DECL_USER_ALIGN (sinit) = true;
-    }
 
   decl->sinit = sinit;
   return sinit;
@@ -2409,7 +2405,7 @@ aggregate_initializer_decl (AggregateDeclaration *decl)
 tree
 layout_class_initializer (ClassDeclaration *cd)
 {
-  NewExp *ne = NewExp::create (cd->loc, NULL, NULL, cd->type, NULL);
+  NewExp *ne = NewExp::create (cd->loc, NULL, cd->type, NULL);
   ne->type = cd->type;
 
   Expression *e = dmd::ctfeInterpret (ne);
@@ -2423,7 +2419,7 @@ layout_struct_initializer (StructDeclaration *sd)
 {
   StructLiteralExp *sle = StructLiteralExp::create (sd->loc, sd, NULL);
 
-  if (!dmd::fill (sd, sd->loc, *sle->elements, true))
+  if (!sd->fill (sd->loc, *sle->elements, true))
     gcc_unreachable ();
 
   sle->type = sd->type;
