@@ -1,5 +1,5 @@
 /* modules.cc -- D module initialization and termination.
-   Copyright (C) 2013-2025 Free Software Foundation, Inc.
+   Copyright (C) 2013-2024 Free Software Foundation, Inc.
 
 GCC is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@ You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING3.  If not see
 <http://www.gnu.org/licenses/>.  */
 
+#define INCLUDE_MEMORY
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
@@ -143,13 +144,13 @@ get_internal_fn (tree ident, const Visibility &visibility)
       name = IDENTIFIER_POINTER (s);
     }
 
-  FuncDeclaration *fd = dmd::genCfunc (NULL, Type::tvoid,
-				       Identifier::idPool (name));
+  FuncDeclaration *fd = FuncDeclaration::genCfunc (NULL, Type::tvoid,
+						   Identifier::idPool (name));
   fd->isGenerated (true);
-  fd->loc = Loc::singleFilename (mod->srcfile.toChars ());
+  fd->loc = Loc (mod->srcfile.toChars (), 1, 0);
   fd->parent = mod;
   fd->visibility = visibility;
-  fd->semanticRun (PASS::semantic3done);
+  fd->semanticRun = PASS::semantic3done;
 
   return fd;
 }
