@@ -117,6 +117,8 @@ enum iocall
   IOCALL_WRITE_DONE,
   IOCALL_X_INTEGER,
   IOCALL_X_INTEGER_WRITE,
+  IOCALL_X_UNSIGNED,
+  IOCALL_X_UNSIGNED_WRITE,
   IOCALL_X_LOGICAL,
   IOCALL_X_LOGICAL_WRITE,
   IOCALL_X_CHARACTER,
@@ -335,6 +337,14 @@ gfc_build_io_library_fndecls (void)
 	get_identifier (PREFIX("transfer_integer_write")), ". w R . ",
 	void_type_node, 3, dt_parm_type, pvoid_type_node, gfc_int4_type_node);
 
+  iocall[IOCALL_X_UNSIGNED] = gfc_build_library_function_decl_with_spec (
+	get_identifier (PREFIX("transfer_unsigned")), ". w W . ",
+	void_type_node, 3, dt_parm_type, pvoid_type_node, gfc_int4_type_node);
+
+  iocall[IOCALL_X_UNSIGNED_WRITE] = gfc_build_library_function_decl_with_spec (
+	get_identifier (PREFIX("transfer_unsigned_write")), ". w R . ",
+	void_type_node, 3, dt_parm_type, pvoid_type_node, gfc_int4_type_node);
+
   iocall[IOCALL_X_LOGICAL] = gfc_build_library_function_decl_with_spec (
 	get_identifier (PREFIX("transfer_logical")), ". w W . ",
 	void_type_node, 3, dt_parm_type, pvoid_type_node, gfc_int4_type_node);
@@ -1040,9 +1050,7 @@ io_result (stmtblock_t * block, tree var, gfc_st_label * err_label,
 static void
 set_error_locus (stmtblock_t * block, tree var, locus * where)
 {
-  gfc_file *f;
   tree str, locus_file;
-  int line;
   gfc_st_parameter_field *p = &st_parameter_field[IOPARM_common_filename];
 
   locus_file = fold_build3_loc (input_location, COMPONENT_REF,
@@ -1051,14 +1059,12 @@ set_error_locus (stmtblock_t * block, tree var, locus * where)
   locus_file = fold_build3_loc (input_location, COMPONENT_REF,
 				TREE_TYPE (p->field), locus_file,
 				p->field, NULL_TREE);
-  f = where->lb->file;
-  str = gfc_build_cstring_const (f->filename);
-
+  location_t loc = gfc_get_location (where);
+  str = gfc_build_cstring_const (LOCATION_FILE (loc));
   str = gfc_build_addr_expr (pchar_type_node, str);
   gfc_add_modify (block, locus_file, str);
 
-  line = LOCATION_LINE (where->lb->location);
-  set_parameter_const (block, var, IOPARM_common_line, line);
+  set_parameter_const (block, var, IOPARM_common_line, LOCATION_LINE (loc));
 }
 
 
@@ -2342,6 +2348,15 @@ transfer_expr (gfc_se * se, gfc_typespec * ts, tree addr_expr,
 
       break;
 
+    case BT_UNSIGNED:
+      arg2 = build_int_cst (unsigned_type_node, kind);
+      if (last_dt == READ)
+	function = iocall[IOCALL_X_UNSIGNED];
+      else
+	function = iocall[IOCALL_X_UNSIGNED_WRITE];
+
+      break;
+
     case BT_REAL:
       arg2 = build_int_cst (integer_type_node, kind);
       if (last_dt == READ)
@@ -2463,8 +2478,8 @@ transfer_expr (gfc_se * se, gfc_typespec * ts, tree addr_expr,
 		  || (ts->type == BT_CLASS
 		      && !GFC_CLASS_TYPE_P (TREE_TYPE (decl))))
 		gfc_conv_derived_to_class (se, code->expr1,
-					   dtio_sub->formal->sym->ts,
-					   vptr, false, false);
+					   dtio_sub->formal->sym, vptr, false,
+					   false, "transfer");
 	      addr_expr = se->expr;
 	      function = iocall[IOCALL_X_DERIVED];
 	      break;
@@ -2630,9 +2645,7 @@ gfc_trans_transfer (gfc_code * code)
 	 && ((expr->symtree->n.sym->ts.type == BT_DERIVED && expr->ts.deferred)
 	     || (expr->symtree->n.sym->assoc
 		 && expr->symtree->n.sym->assoc->variable)
-	     || gfc_expr_attr (expr).pointer
-	     || (expr->symtree->n.sym->attr.pointer
-		 && gfc_expr_attr (expr).target)))
+	     || gfc_expr_attr (expr).pointer))
 	goto scalarize;
 
       /* With array-bounds checking enabled, force scalarization in some
