@@ -1,5 +1,5 @@
 /* Harden conditionals.
-   Copyright (C) 2021-2023 Free Software Foundation, Inc.
+   Copyright (C) 2021-2024 Free Software Foundation, Inc.
    Contributed by Alexandre Oliva <oliva@adacore.com>.
 
 This file is part of GCC.
@@ -18,6 +18,7 @@ You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING3.  If not see
 <http://www.gnu.org/licenses/>.  */
 
+#define INCLUDE_MEMORY
 #include "config.h"
 #include "system.h"
 #include "coretypes.h"
@@ -235,6 +236,7 @@ insert_check_and_trap (location_t loc, gimple_stmt_iterator *gsip,
   gsi_insert_before (gsip, cond, GSI_SAME_STMT);
 
   basic_block trp = create_empty_bb (chk);
+  trp->count = profile_count::zero ();
 
   gimple_stmt_iterator gsit = gsi_after_labels (trp);
   gcall *trap = gimple_build_call (builtin_decl_explicit (BUILT_IN_TRAP), 0);
@@ -276,8 +278,8 @@ insert_check_and_trap (location_t loc, gimple_stmt_iterator *gsip,
 }
 
 /* Split edge E, and insert_check_and_trap (see above) in the
-   newly-created block, using detached copies of LHS's and RHS's
-   values (see detach_value above) for the COP compare.  */
+   newly-created block, using already-detached copies of LHS's and
+   RHS's values (see detach_value above) for the COP compare.  */
 
 static inline void
 insert_edge_check_and_trap (location_t loc, edge e,
@@ -301,10 +303,6 @@ insert_edge_check_and_trap (location_t loc, edge e,
 
   gimple_stmt_iterator gsik = gsi_after_labels (chk);
 
-  bool same_p = (lhs == rhs);
-  lhs = detach_value (loc, &gsik, lhs);
-  rhs = same_p ? lhs : detach_value (loc, &gsik, rhs);
-
   insert_check_and_trap (loc, &gsik, flags, cop, lhs, rhs);
 }
 
@@ -366,6 +364,12 @@ pass_harden_conditional_branches::execute (function *fun)
 	/* ??? Can we do better?  */
 	continue;
 
+      /* Detach the values before the compares.  If we do so later,
+	 the compiler may use values inferred from the compares.  */
+      bool same_p = (lhs == rhs);
+      lhs = detach_value (loc, &gsi, lhs);
+      rhs = same_p ? lhs : detach_value (loc, &gsi, rhs);
+
       insert_edge_check_and_trap (loc, EDGE_SUCC (bb, 0), cop, lhs, rhs);
       insert_edge_check_and_trap (loc, EDGE_SUCC (bb, 1), cop, lhs, rhs);
     }
@@ -508,6 +512,13 @@ pass_harden_compares::execute (function *fun)
 
 	  tree rhs = copy_ssa_name (lhs);
 
+	  /* Detach the values before the compares, so that the
+	     compiler infers nothing from them, not even from a
+	     throwing compare that didn't throw.  */
+	  bool same_p = (op1 == op2);
+	  op1 = detach_value (loc, &gsi, op1);
+	  op2 = same_p ? op1 : detach_value (loc, &gsi, op2);
+
 	  gimple_stmt_iterator gsi_split = gsi;
 	  /* Don't separate the original assignment from debug stmts
 	     that might be associated with it, and arrange to split the
@@ -529,10 +540,6 @@ pass_harden_compares::execute (function *fun)
 			 gimple_bb (asgn)->index, nbb->index);
 	    }
 
-	  bool same_p = (op1 == op2);
-	  op1 = detach_value (loc, &gsi_split, op1);
-	  op2 = same_p ? op1 : detach_value (loc, &gsi_split, op2);
-
 	  gassign *asgnck = gimple_build_assign (rhs, cop, op1, op2);
 	  gimple_set_location (asgnck, loc);
 	  gsi_insert_before (&gsi_split, asgnck, GSI_SAME_STMT);
@@ -574,11 +581,21 @@ pass_harden_compares::execute (function *fun)
 	  if (throwing_compare_p)
 	    {
 	      add_stmt_to_eh_lp (asgnck, lookup_stmt_eh_lp (asgn));
-	      make_eh_edges (asgnck);
+	      edge eh = make_eh_edge (asgnck);
+	      /* This compare looks like it could raise an exception,
+		 but it's dominated by the original compare, that
+		 would raise an exception first, so the EH edge from
+		 this one is never really taken.  */
+	      eh->probability = profile_probability::never ();
+	      if (eh->dest->count.initialized_p ())
+		eh->dest->count += eh->count ();
+	      else
+		eh->dest->count = eh->count ();
 
 	      edge ckeh;
 	      basic_block nbb = split_edge (non_eh_succ_edge
 					    (gimple_bb (asgnck), &ckeh));
+	      gcc_checking_assert (eh == ckeh);
 	      gsi_split = gsi_start_bb (nbb);
 
 	      if (dump_file)
