@@ -1,6 +1,6 @@
 /* gm2-lang.cc language-dependent hooks for GNU Modula-2.
 
-Copyright (C) 2002-2023 Free Software Foundation, Inc.
+Copyright (C) 2002-2025 Free Software Foundation, Inc.
 Contributed by Gaius Mulley <gaius@glam.ac.uk>.
 
 This file is part of GNU Modula-2.
@@ -16,9 +16,8 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Modula-2; see the file COPYING.  If not, write to the
-Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.  */
+along with GCC; see the file COPYING3.  If not see
+<http://www.gnu.org/licenses/>.  */
 
 #define INCLUDE_VECTOR
 #include "gm2-gcc/gcc-consolidation.h"
@@ -131,7 +130,7 @@ gm2_langhook_init (void)
 
   if (M2Options_GetPPOnly ())
     {
-      /* preprocess the file here.  */
+      /* Preprocess the file here.  */
       gm2_langhook_parse_file ();
       return false; /* Finish now, no further compilation.  */
     }
@@ -191,7 +190,8 @@ gm2_langhook_init_options (unsigned int decoded_options_count,
       switch (code)
 	{
 	case OPT_fcpp:
-	  gcc_checking_assert (building_cpp_command);
+	  if (value)
+	    gcc_checking_assert (building_cpp_command);
 	  break;
 	case OPT_fcpp_begin:
 	  in_cpp_args = true;
@@ -214,8 +214,7 @@ gm2_langhook_init_options (unsigned int decoded_options_count,
 	  M2Options_Setc (value);
 	  break;
 	case OPT_dumpdir:
-	  if (building_cpp_command)
-	    M2Options_SetDumpDir (arg);
+	  M2Options_SetDumpDir (arg);
 	  break;
 	case OPT_save_temps:
 	  if (building_cpp_command)
@@ -234,23 +233,54 @@ gm2_langhook_init_options (unsigned int decoded_options_count,
 	      building_cpp_command = true;
 	    }
 	  M2Options_CppArg (opt, arg, (option->flags & CL_JOINED)
-			      && !(option->flags & CL_SEPARATE));
-	  break;
-	case OPT_M:
-	case OPT_MM:
-	  gcc_checking_assert (building_cpp_command);
-	  M2Options_SetPPOnly (value);
-	  /* This is a preprocessor command.  */
-	  M2Options_CppArg (opt, arg, (option->flags & CL_JOINED)
-			      && !(option->flags & CL_SEPARATE));
+			    && !(option->flags & CL_SEPARATE));
 	  break;
 
-	/* We can only use MQ when the command line is either PP-only, or
+	case OPT_M:
+	  /* Output a rule suitable for make describing the dependencies of the
+	     main source file.  */
+	  if (in_cpp_args)
+	    {
+	      gcc_checking_assert (building_cpp_command);
+	      /* This is a preprocessor command.  */
+	      M2Options_CppArg (opt, arg, (option->flags & CL_JOINED)
+				&& !(option->flags & CL_SEPARATE));
+	    }
+	  M2Options_SetPPOnly (value);
+	  M2Options_SetM (value);
+	  break;
+
+	case OPT_MM:
+	  if (in_cpp_args)
+	    {
+	      gcc_checking_assert (building_cpp_command);
+	      /* This is a preprocessor command.  */
+	      M2Options_CppArg (opt, arg, (option->flags & CL_JOINED)
+				&& !(option->flags & CL_SEPARATE));
+	    }
+	  M2Options_SetPPOnly (value);
+	  M2Options_SetMM (value);
+	  break;
+
+	case OPT_MF:
+	  if (!in_cpp_args)
+	    M2Options_SetMF (arg);
+	  break;
+
+	case OPT_MP:
+	  M2Options_SetMP (value);
+	  break;
+
+	/* We can only use MQ and MT when the command line is either PP-only, or
 	   when there is a MD/MMD on it.  */
 	case OPT_MQ:
 	  M2Options_SetMQ (arg);
 	  break;
 
+	case OPT_MT:
+	  M2Options_SetMT (arg);
+	  break;
+
 	case OPT_o:
 	  M2Options_SetObj (arg);
 	  break;
@@ -266,14 +296,23 @@ gm2_langhook_init_options (unsigned int decoded_options_count,
 	     For now skip all plugins to avoid fails with the m2 one.  */
 	  break;
 
-	/* Preprocessor arguments with a following filename, we add these
-	   back to the main file preprocess line, but not to dependents
-	   TODO Handle MF.  */
+	/* Preprocessor arguments with a following filename.  */
 	case OPT_MD:
-	  M2Options_SetMD (arg);
+	  M2Options_SetMD (value);
+	  if (value)
+	    {
+	      M2Options_SetM (true);
+	      M2Options_SetMF (arg);
+	    }
 	  break;
+
 	case OPT_MMD:
-	  M2Options_SetMMD (arg);
+	  M2Options_SetMMD (value);
+	  if (value)
+	    {
+	      M2Options_SetMM (true);
+	      M2Options_SetMF (arg);
+	    }
 	  break;
 
 	/* Modula 2 claimed options we pass to the preprocessor.  */
@@ -367,6 +406,9 @@ gm2_langhook_handle_option (
 
   switch (code)
     {
+    case OPT_dumpdir:
+      M2Options_SetDumpDir (arg);
+      return 1;
     case OPT_I:
       push_back_Ipath (arg);
       return 1;
@@ -427,11 +469,8 @@ gm2_langhook_handle_option (
     case OPT_fd:
       M2Options_SetCompilerDebugging (value);
       return 1;
-    case OPT_fdebug_trace_quad:
-      M2Options_SetDebugTraceQuad (value);
-      return 1;
-    case OPT_fdebug_trace_api:
-      M2Options_SetDebugTraceAPI (value);
+    case OPT_fdebug_builtins:
+      M2Options_SetDebugBuiltins (value);
       return 1;
     case OPT_fdebug_function_line_numbers:
       M2Options_SetDebugFunctionLineNumbers (value);
@@ -476,9 +515,29 @@ gm2_langhook_handle_option (
     case OPT_fm2_strict_type:
       M2Options_SetStrictTypeChecking (value);
       return 1;
+    case OPT_fm2_debug_trace_:
+      M2Options_SetM2DebugTraceFilter (value, arg);
+      return 1;
+    case OPT_fm2_dump_:
+      return M2Options_SetM2Dump (value, arg);
+    case OPT_fm2_dump_decl_:
+      M2Options_SetDumpDeclFilename (value, arg);
+      return 1;
+    case OPT_fm2_dump_gimple_:
+      M2Options_SetDumpGimpleFilename (value, arg);
+      return 1;
+    case OPT_fm2_dump_quad_:
+      M2Options_SetDumpQuadFilename (value, arg);
+      return 1;
+    case OPT_fm2_dump_filter_:
+      M2Options_SetM2DumpFilter (value, arg);
+      return 1;
     case OPT_Wall:
       M2Options_SetWall (value);
       return 1;
+    case OPT_Wcase_enum:
+      M2Options_SetCaseEnumChecking (value);
+      return 1;
 #if 0
     /* Not yet implemented.  */
     case OPT_fxcode:
@@ -593,6 +652,16 @@ gm2_langhook_handle_option (
     case OPT_fm2_whole_program:
       M2Options_SetWholeProgram (value);
       return 1;
+#ifdef OPT_mabi_ibmlongdouble
+    case OPT_mabi_ibmlongdouble:
+      M2Options_SetIBMLongDouble (value);
+      return 1;
+#endif
+#ifdef OPT_mabi_ieeelongdouble
+    case OPT_mabi_ieeelongdouble:
+      M2Options_SetIEEELongDouble (value);
+      return 1;
+#endif
     case OPT_flocation_:
       if (strcmp (arg, "builtins") == 0)
         {
@@ -738,7 +807,7 @@ gm2_langhook_post_options (const char **pfilename)
   if (allow_libraries)
     add_m2_import_paths (flibs);
 
- /* Returning false means that the backend should be used.  */
+  /* Returning false means that the backend should be used.  */
   return M2Options_GetPPOnly ();
 }
 
@@ -806,14 +875,25 @@ gm2_langhook_type_for_mode (machine_mode mode, int unsignedp)
   if (mode == TYPE_MODE (long_double_type_node))
     return long_double_type_node;
 
+  if ((float128_type_node != NULL) && (mode == TYPE_MODE (float128_type_node)))
+    return float128_type_node;
+
   if (COMPLEX_MODE_P (mode))
     {
+      machine_mode inner_mode;
+      tree inner_type;
+
       if (mode == TYPE_MODE (complex_float_type_node))
 	return complex_float_type_node;
       if (mode == TYPE_MODE (complex_double_type_node))
 	return complex_double_type_node;
       if (mode == TYPE_MODE (complex_long_double_type_node))
 	return complex_long_double_type_node;
+
+      inner_mode = GET_MODE_INNER (mode);
+      inner_type = gm2_langhook_type_for_mode (inner_mode, unsignedp);
+      if (inner_type != NULL_TREE)
+	return build_complex_type (inner_type);
     }
 
 #if HOST_BITS_PER_WIDE_INT >= 64
